/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.flamenk.dom;

/**
 * A {@link HtmlNode} ranker, scores a node against
 * the possibility to contain an article.
 *
 * There is no guarantee that when node ranker is called the tree is entirely built.
 * With the current implementation, the node ranker is invoked when
 * <lu>
 *  <li>All the node's children have been built.</li>
 *  <li>All the node's left siblings have been built.</li>
 *  <li>All the node's right siblings have not been built.</li>
 *  <li>For each node's ancestor only the left siblings have been built.</li>
 * </lu>
 *
 * @author Torindo Nesci
 */
public interface HtmlNodeArticleRanker {

  /**
   * Ranks the given {@link HtmlNode}.
   *
   * @param node The html node.
   * @param tokenizer The tokenizer used to build the node terms.
   * @return The html node article rank.
   */
  double rank(HtmlNode node, TextTokenizer tokenizer);
  
}
