/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.flamenk.histogram;

import com.flamenk.TagConstants;
import com.flamenk.util.TokenUtil;
import static com.flamenk.util.TokenUtil.hasName;
import com.google.common.base.Preconditions;

/**
 * A simple token ranker implementation.
 *
 * <br/><br/> <b>This implementation is Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public final class SimpleTokenRanker implements TokenRanker {

  private static final SimpleTokenRanker INSTANCE =
          new SimpleTokenRanker();
  
  private SimpleTokenRanker() {
  
  }
  
  /**
   * Returns the singleton instance.
   *
   * @return The html token ranker.
   */
  public static SimpleTokenRanker getInstance() {
    return INSTANCE;
  }
  
  @Override
  public double rank(Token token) {
    Preconditions.checkNotNull(token);
    
    if (hasName(token, TagConstants.TEXT)
            || TokenUtil.isHeaderToken(token)
            || isTextFormatter(token)
            || hasName(token, TagConstants.P)
            || hasName(token, TagConstants.BR)
            || hasName(token, TagConstants.SPAN)) {
      return 1;
    } else if (TokenUtil.isTableToken(token)) {
      return 0.65D;
    } else if (hasName(token, TagConstants.A)) {
      return 0.4D;
    } else if (hasName(token, TagConstants.LI)
            || hasName(token, TagConstants.UL)) {
      return 0.8D;
    }
    return 0D;
  }

  private boolean isTextFormatter(Token token) {
    return hasName(token, TagConstants.B)
            || hasName(token, TagConstants.I)
            || hasName(token, TagConstants.EM)
            || hasName(token, TagConstants.STRONG)
            || hasName(token, TagConstants.PRE)
            || hasName(token, TagConstants.CODE)
            || hasName(token, TagConstants.SUP);
  }
  
}
