/*
 * Copyright 2013 Torindo Nesci.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.flamenk.util;

import com.google.common.base.Preconditions;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/**
 * An immutable List. This implementation does not guarantee to be immutable,
 * since the backed list can be modified. In order to guarantee the immutability
 * of the current ImmutableList, the backed list must not be exposed to the
 * client. <br/> All the methods calls are forwarded to the backed list.
 *
 * <br/><br/><b>This implementation is Thread Safe only if the backed List is
 * Thread Safe.</b>
 *
 * @author Torindo Nesci
 */
public class ImmutableList<T> implements List<T> {

  private final List<T> mList;

  /**
   * Creates a new immutable list.
   *
   * @param list The list to forward the calls to.
   */
  public ImmutableList(List<T> list) {
    Preconditions.checkNotNull(list);

    this.mList = list;
  }

  @Override
  public int size() {
    return this.mList.size();
  }

  @Override
  public boolean isEmpty() {
    return this.mList.isEmpty();
  }

  @Override
  public boolean contains(Object o) {
    return this.mList.contains(o);
  }

  @Override
  public Iterator<T> iterator() {
    return this.mList.iterator();
  }

  @Override
  public Object[] toArray() {
    return this.mList.toArray();
  }

  @Override
  public <T> T[] toArray(T[] a) {
    return this.mList.toArray(a);
  }

  /**
   * Not implemented.
   *
   * @param e
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public boolean add(T e) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @param o
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public boolean remove(Object o) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public boolean containsAll(Collection<?> c) {
    return this.mList.containsAll(c);
  }

  /**
   * Not implemented.
   *
   * @param c
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public boolean addAll(Collection<? extends T> c) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @param index
   * @param c
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public boolean addAll(int index, Collection<? extends T> c) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @param c
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public boolean removeAll(Collection<?> c) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @param c
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public boolean retainAll(Collection<?> c) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public void clear() {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public T get(int index) {
    return this.mList.get(index);
  }

  /**
   * Not implemented.
   *
   * @param index
   * @param element
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public T set(int index, T element) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @param index
   * @param element
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public void add(int index, T element) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @param index
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public T remove(int index) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public int indexOf(Object o) {
    return this.mList.indexOf(o);
  }

  @Override
  public int lastIndexOf(Object o) {
    return this.mList.lastIndexOf(o);
  }

  /**
   * Not implemented.
   *
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public ListIterator<T> listIterator() {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @param index
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public ListIterator<T> listIterator(int index) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  /**
   * Not implemented.
   *
   * @param fromIndex
   * @param toIndex
   * @throws UnsupportedOperationException Always.
   */
  @Override
  public List<T> subList(int fromIndex, int toIndex) {
    throw new UnsupportedOperationException("Not supported yet.");
  }
}
